@@ -3,6 +3,7 @@ from __future__ import annotations
 import os
 import ctypes
 import json
+import urllib
 import logging
 import webbrowser
 from collections.abc import Callable
@@ -16,7 +17,6 @@ from PyObjCTools import AppHelper
 
 from webview import (FOLDER_DIALOG, OPEN_DIALOG, SAVE_DIALOG, _settings, parse_file_type, windows, settings as webview_settings)
 from webview.dom import _dnd_state
-from webview.js.css import disable_text_select
 from webview.menu import Menu, MenuAction, MenuSeparator
 from webview.screen import Screen
 from webview.util import DEFAULT_HTML, create_cookie, js_bridge_call, inject_pywebview
@@ -94,19 +94,19 @@ class BrowserView:
             if i.pywebview_window in windows:
                 windows.remove(i.pywebview_window)
 
-            i.webkit.setNavigationDelegate_(None)
-            i.webkit.setUIDelegate_(None)
+            i.webview.setNavigationDelegate_(None)
+            i.webview.setUIDelegate_(None)
 
             # this seems to be a bug in WkWebView, so we need to load blank html
             # see https://stackoverflow.com/questions/27410413/wkwebview-embed-video-keeps-playing-sound-after-release
-            i.webkit.loadHTMLString_baseURL_('', None)
-            i.webkit.removeFromSuperview()
-            i.webkit = None
+            i.webview.loadHTMLString_baseURL_('', None)
+            i.webview.removeFromSuperview()
+            i.webview = None
 
             i.closed.set()
-
             if BrowserView.instances == {}:
                 BrowserView.app.stop_(self)
+                BrowserView.app.abortModal_()
 
         def windowDidResize_(self, notification):
             i = BrowserView.get_instance('window', notification.object())
@@ -191,7 +191,7 @@ class BrowserView:
         def webView_runJavaScriptConfirmPanelWithMessage_initiatedByFrame_completionHandler_(
             self, webview, message, frame, handler
         ):
-            i = BrowserView.get_instance('webkit', webview)
+            i = BrowserView.get_instance('webview', webview)
             ok = i.localization['global.ok']
             cancel = i.localization['global.cancel']
 
@@ -295,7 +295,7 @@ class BrowserView:
         # Show the webview when it finishes loading
         def webView_didFinishNavigation_(self, webview, nav):
             # Add the webview to the window if it's not yet the contentView
-            i = BrowserView.get_instance('webkit', webview)
+            i = BrowserView.get_instance('webview', webview)
 
             if i:
                 if not webview.window():
@@ -303,23 +303,14 @@ class BrowserView:
                     i.window.makeFirstResponder_(webview)
 
                 script = inject_pywebview(i.js_bridge.window, 'cocoa')
-                i.webkit.evaluateJavaScript_completionHandler_(script, lambda a, b: None)
-
-                if not i.text_select:
-                    i.webkit.evaluateJavaScript_completionHandler_(
-                        disable_text_select, lambda a, b: None
-                    )
-
-                print_hook = 'window.print = function() { window.webkit.messageHandlers.browserDelegate.postMessage("print") };'
-                i.webkit.evaluateJavaScript_completionHandler_(print_hook, lambda a, b: None)
-
+                i.webview.evaluateJavaScript_completionHandler_(script, lambda a, b: None)
                 i.loaded.set()
 
         # Handle JavaScript window.print()
         def userContentController_didReceiveScriptMessage_(self, controller, message):
             if message.body() == 'print':
                 i = BrowserView.get_instance('_browserDelegate', self)
-                BrowserView.print_webview(i.webkit)
+                BrowserView.print_webview(i.webview)
 
     class FileFilterChooser(AppKit.NSPopUpButton):
         def initWithFilter_(self, file_filter):
@@ -331,9 +322,12 @@ class BrowserView:
             self.setTarget_(self)
             return self
 
+        def setFileDialog_(self, file_dlg):
+            self.file_dlg = file_dlg
+
         def onChange_(self, sender):
             option = sender.indexOfSelectedItem()
-            self.window().setAllowedFileTypes_(self.filter[option][1])
+            self.file_dlg.setAllowedFileTypes_(self.filter[option][1])
 
     class WebKitHost(WebKit.WKWebView):
         def performDragOperation_(self, sender):
@@ -345,10 +339,10 @@ class BrowserView:
                 }
                 urls = pboard.readObjectsForClasses_options_(classes, options) or []
                 files = [
-                    (os.path.basename(url.filePathURL().absoluteString()), url.filePathURL().absoluteString().replace('file://', ''))
-                    for url
-                    in urls
-                    if url.filePathURL().absoluteString().startswith('file://')
+                    (os.path.basename(os.path.dirname(file_path)) if os.path.isdir(file_path) else os.path.basename(file_path),file_path)
+                    for url in urls
+                    for file_path in [urllib.parse.unquote(url.filePathURL().absoluteString().replace('file://', ''))]
+                    if os.path.isdir(file_path) or os.path.isfile(file_path)
                 ]
 
                 _dnd_state['paths'] += files
@@ -356,7 +350,7 @@ class BrowserView:
             return super(BrowserView.WebKitHost, self).performDragOperation_(sender)
 
         def mouseDown_(self, event):
-            i = BrowserView.get_instance('webkit', self)
+            i = BrowserView.get_instance('webview', self)
             window = self.window()
 
             if i.frameless and i.easy_drag:
@@ -371,7 +365,7 @@ class BrowserView:
             super(BrowserView.WebKitHost, self).mouseDown_(event)
 
         def mouseDragged_(self, event):
-            i = BrowserView.get_instance('webkit', self)
+            i = BrowserView.get_instance('webview', self)
             window = self.window()
 
             if i.frameless and i.easy_drag:
@@ -399,7 +393,7 @@ class BrowserView:
                 window.setFrameOrigin_(newOrigin)
 
             if event.modifierFlags() & getattr(AppKit, 'NSEventModifierFlagControl', 1 << 18):
-                i = BrowserView.get_instance('webkit', self)
+                i = BrowserView.get_instance('webview', self)
                 if not _settings['debug']:
                     return
 
@@ -458,7 +452,6 @@ class BrowserView:
         self.loaded = window.events.loaded
         self.confirm_close = window.confirm_close
         self.title = window.title
-        self.text_select = window.text_select
         self.is_fullscreen = False
         self.hidden = window.hidden
         self.minimized = window.minimized
@@ -496,6 +489,8 @@ class BrowserView:
             )
             .retain()
         )
+        self.pywebview_window.native = self.window
+
         self.window.focus = window.focus
         self.window.setTitle_(window.title)
         self.window.setMinSize_(AppKit.NSSize(window.min_size[0], window.min_size[1]))
@@ -507,19 +502,19 @@ class BrowserView:
         frame.size.height = window.initial_height
         self.window.setFrame_display_(frame, True)
 
-        self.webkit = BrowserView.WebKitHost.alloc().initWithFrame_(rect).retain()
-        self.webkit.pywebview_window = window
+        self.webview = BrowserView.WebKitHost.alloc().initWithFrame_(rect).retain()
+        self.webview.pywebview_window = window
 
         self._browserDelegate = BrowserView.BrowserDelegate.alloc().init().retain()
         self._windowDelegate = BrowserView.WindowDelegate.alloc().init().retain()
         self._appDelegate = BrowserView.AppDelegate.alloc().init().retain()
 
         BrowserView.app.setDelegate_(self._appDelegate)
-        self.webkit.setUIDelegate_(self._browserDelegate)
-        self.webkit.setNavigationDelegate_(self._browserDelegate)
+        self.webview.setUIDelegate_(self._browserDelegate)
+        self.webview.setNavigationDelegate_(self._browserDelegate)
         self.window.setDelegate_(self._windowDelegate)
 
-        config = self.webkit.configuration()
+        config = self.webview.configuration()
         config.userContentController().addScriptMessageHandler_name_(
             self._browserDelegate, 'browserDelegate'
         )
@@ -556,7 +551,7 @@ class BrowserView:
 
         user_agent = webview_settings.get('user_agent') or _settings['user_agent']
         if user_agent:
-            self.webkit.setCustomUserAgent_(user_agent)
+            self.webview.setCustomUserAgent_(user_agent)
 
         self.window.setFrameOrigin_(self.screen.origin)
 
@@ -571,7 +566,7 @@ class BrowserView:
             self.window.setBackgroundColor_(
                 BrowserView.nscolor_from_hex(window.background_color, 0)
             )
-            self.webkit.setValue_forKey_(True, 'drawsTransparentBackground')
+            self.webview.setValue_forKey_(True, 'drawsTransparentBackground')
         else:
             self.window.setBackgroundColor_(BrowserView.nscolor_from_hex(window.background_color))
 
@@ -585,8 +580,8 @@ class BrowserView:
             visualEffectView.setFrame_(frame_vibrancy)
             visualEffectView.setState_(AppKit.NSVisualEffectStateActive)
             visualEffectView.setBlendingMode_(AppKit.NSVisualEffectBlendingModeBehindWindow)
-            self.webkit.addSubview_positioned_relativeTo_(
-                visualEffectView, AppKit.NSWindowBelow, self.webkit
+            self.webview.addSubview_positioned_relativeTo_(
+                visualEffectView, AppKit.NSWindowBelow, self.webview
             )
 
         self.frameless = window.frameless
@@ -608,8 +603,10 @@ class BrowserView:
         if window.on_top:
             self.window.setLevel_(AppKit.NSStatusWindowLevel)
 
+        self.pywebview_window.events.before_show.set()
+
         try:
-            self.webkit.evaluateJavaScript_completionHandler_('', lambda a, b: None)
+            self.webview.evaluateJavaScript_completionHandler_('', lambda a, b: None)
         except TypeError:
             registerMetaDataForSelector(
                 b'WKWebView', b'evaluateJavaScript:completionHandler:', _eval_js_metadata
@@ -624,7 +621,6 @@ class BrowserView:
             self.load_html(DEFAULT_HTML, '')
         if window.fullscreen:
             self.toggle_fullscreen()
-        self.shown.set()
 
     def first_show(self):
         if not self.hidden:
@@ -635,6 +631,8 @@ class BrowserView:
         elif self.minimized:
             self.minimize()
 
+        self.shown.set()
+
         if not BrowserView.app.isRunning():
             # Reset the application menu to the defaults
             self._clear_main_menu()
@@ -768,7 +766,7 @@ class BrowserView:
 
     def get_current_url(self):
         def get():
-            self._current_url = str(self.webkit.URL())
+            self._current_url = str(self.webview.URL())
             self._current_url_semaphore.release()
 
         AppHelper.callAfter(get)
@@ -780,7 +778,7 @@ class BrowserView:
         def load(url):
             page_url = Foundation.NSURL.URLWithString_(BrowserView.quote(url))
             req = Foundation.NSURLRequest.requestWithURL_(page_url)
-            self.webkit.loadRequest_(req)
+            self.webview.loadRequest_(req)
 
         self.loaded.clear()
         self.url = url
@@ -789,14 +787,14 @@ class BrowserView:
     def load_html(self, content, base_uri):
         def load(content, url):
             url = Foundation.NSURL.URLWithString_(BrowserView.quote(url))
-            self.webkit.loadHTMLString_baseURL_(content, url)
+            self.webview.loadHTMLString_baseURL_(content, url)
 
         self.loaded.clear()
         AppHelper.callAfter(load, content, base_uri)
 
     def evaluate_js(self, script):
         def eval():
-            self.webkit.evaluateJavaScript_completionHandler_(script, handler)
+            self.webview.evaluateJavaScript_completionHandler_(script, handler)
 
         def handler(result, error):
             JSResult.result = None if result is None else json.loads(result)
@@ -857,6 +855,7 @@ class BrowserView:
                         filter_chooser = BrowserView.FileFilterChooser.alloc().initWithFilter_(
                             file_filter
                         )
+                        filter_chooser.setFileDialog_(open_dlg)
                         open_dlg.setAccessoryView_(filter_chooser)
                         open_dlg.setAccessoryViewDisclosed_(True)
 
@@ -1117,6 +1116,7 @@ def create_window(window):
     if window.uid == 'master':
         main_thread().pydev_do_not_trace = True # vs code debugger hang fix
         create()
+
     else:
         AppHelper.callAfter(create)
 
@@ -1154,7 +1154,7 @@ def create_file_dialog(dialog_type, directory, allow_multiple, save_filename, fi
     for s in file_types:
         description, extensions = parse_file_type(s)
         file_extensions = [i.lstrip('*.') for i in extensions.split(';') if i != '*.*']
-        file_filter.append([description, file_extensions or None])
+        file_filter.append([description, file_extensions or []])
 
     i = BrowserView.instances.get(uid)
     return i.create_file_dialog(dialog_type, directory, allow_multiple, save_filename, file_filter)
